import sys
import click
from loinchpo import AnnotationParser
from loinchpo import QueryResolver
from loinchpo import QueryFileParser

@click.group()
def cli():
    pass


@cli.command()
@click.argument('annotation_path', type=click.Path(exists=True))
@click.argument('query_path', type=click.Path(exists=True))
def resolve(annotation_path, query_path):
    """Command to resolve queries to hpo codes
    Todo:
        * Need to make this better reporting to a file
    Args:
        annotation_path (str): is the path to the LOINC HPO Annotation File.
        query_path (str): is the path to your file with Loinc Id's, Measures, Negations.
    """
    click.echo("Parsing annotation files...")
    click.echo(click.format_filename(annotation_path))
    click.echo(click.format_filename(query_path))
    annotations = AnnotationParser.parse_annotation_file(annotation_path)
    queries = QueryFileParser.parse(query_path)
    resolver = QueryResolver(annotations)
    click.echo("Resolving your queries...")
    for query in queries:
        result = resolver.resolve(query)
        print(result)
    click.echo("Done.")


@cli.command()
def version():
    """Command to check the version of LOINC2HPO"""
    click.echo("Version not configured yet.")


if __name__ == '__main__':
    sys.exit(cli())



import pyspark.sql.functions as F


class Cleaner:
    @staticmethod
    def impute_measurement_values(df):
        """As indicated by the OMOP CDM, make sure that all negative values coming from the source, set the value_as_number to NULL,
        with the exception of the following Measurements:
            - 3003396 (Base excess in Arterial blood by calculation)
            - 3002032 (Base excess in Venous blood by calculation)
            - 3006277 (QRS-Axis), 3012501 (Base excess in Blood by calculation)
            - 3003129 (Base excess in Capillary blood by calculation)
            - 3004959 (Base excess in Arterial cord blood by calculation)
            - 3007435 (Base excess in Venous cord blood by calculation)
        """

        concept_list = [3003396, 3002032, 3006277, 3012501, 3003129, 3004959, 3007435]
        updated_values = df.withColumn("value_as_number",
                                       F.when((df.value_as_number < 0.0) & (~df.concept_id.isin(concept_list)),
                                              None).otherwise(df.value_as_number))

        return updated_values



import pyspark.sql.functions as F


class ConceptSynonymTransformer:

    def transform(self, df, concept, concept_synonym):
        """
            A function to attach synonyms to loinc concept_names.
        """
        language_concept = self.english_concept_synonym_filter(concept_synonym, concept.select("concept_id", "concept_name").dropDuplicates())
        concept_synonym = concept_synonym.filter(concept_synonym.language_concept_id == language_concept.concept_id)
        concept_synonym = self.aggregate_synonyms_by_concept(concept_synonym).drop(
            *["concept_synonym_name"]).dropDuplicates()
        return df.join(concept_synonym, ["concept_id"], "left")

    def english_concept_synonym_filter(self, concept_synonym, concept):
        """
            Filter the concept table down to only the concepts that are in concept_synonym
            then join and find the english one
        """
        language_concept_ids = set(concept_synonym.select("language_concept_id").toPandas()["language_concept_id"])
        english = concept.where(F.col("concept_id").isin(language_concept_ids)).where(
            F.lower(F.col("concept_name")).contains("english"))
        return english.first()

    def aggregate_synonyms_by_concept(self, df):
        """
            Aggregates synonyms by concept as a comma-delimited string
        """

        df = df.na.fill(value="None").dropDuplicates()
        df = df.groupBy("concept_id").agg(F.collect_list("concept_synonym_name").alias("synonym_list")).dropDuplicates()
        df = df.withColumn("synonym_list", F.lower(F.array_join("synonym_list", ",")))
        return df



from pyspark.sql.functions import udf
from pyspark.sql.types import StringType

from loinchpo import ConceptSynonymTransformer
from loinchpo import MeasurementTransformer
from loinchpo import LoincScale


class OMOPTransformer:

    @staticmethod
    def transform(measurement_table, concept_table, concept_synonym_table):
        m = MeasurementTransformer()
        df = m.transform(measurement_table, concept_table)
        s = ConceptSynonymTransformer()
        df = s.transform(df, concept_table, concept_synonym_table)
        loinc_scale_udf = udf(lambda x: LoincScale.infer_long(x), StringType())
        df = df.select("*", loinc_scale_udf("synonym_list").alias("loinc_scale_type")).dropDuplicates()
        return df.select("person_id", "visit_occurrence_id", "measurement_id",
                         "measurement_date", "concept_id", "concept_code", "concept_name", "loinc_result",
                         "loinc_scale_type", "value_as_concept_name", "value_as_number", "range_low", "range_high")



from loinchpo import Utility
from loinchpo.error.LoincHpoFileValidationError import LoincHpoValidationError


class Query:
    def __init__(self, loinc_id, outcome):
        try:
            Utility.check_all(loinc_id, outcome)
            self.loinc_id = loinc_id
            self.outcome = Utility.parse_outcome(outcome)
        except LoincHpoValidationError as e:
            raise e



import csv
from loinchpo.error.LoincHpoParsingError import LoincHpoParsingError
from loinchpo.error.LoincHpoFileValidationError import LoincHpoValidationError
from loinchpo import Utility
from loinchpo import LoincScale
from collections import namedtuple

LoincHpoAnnotation = namedtuple('LoincHpoAnnotation',
                                'loinc_id, loinc_scale, outcome, hpo_term,'
                                'supplemental_term, curators, comment')


class AnnotationParser:
    """ Provides methods for parsing :class:`LoincHpoAnnotation` files.

        Example:
            annotations = AnnotationParser.parse_annotation_file(file_path)
            annotation_map = AnnotationParser.parse_annotation_file_dict(file_path)

    """

    @staticmethod
    def parse_annotation_file(file_path, ls=False):
        """ A traversable dictionary of loinc_id -> loinc_measure -> negation Parsing Loinc2HpoAnnotation files to
        either a list of namedtuples :class:`LoincHpoAnnotation`

        Args:
            file_path: A string file path to the annotation file.
            ls: Should the method return a list of ::class::`LoincHpoAnnotation`

        Returns:
            annotations: A dictionary of loinc_id -> loinc_measure -> negation or a list of namedtuples ::class::`LoincHpoAnnotation`

        Raises:
            OSError: When the file could not be opened or found.
            LoincHpoParsingError: When there were issues with file format
            LoincHpoValidationError: When there were issues with specific field expected values.
        """

        annotations = {}
        if ls:
            annotations = []
        try:
            with open(file_path) as f:
                reader = csv.reader(f, delimiter='\t')
                fields = next(reader)

                if "loincId" not in fields:
                    raise LoincHpoParsingError("Header Line not Loinc2Hpo annotation file.")
                for line in reader:
                    loinc_id, loinc_scale, outcome, hpo_term, supplemental_term, curators, comment = line

                    Utility.check_all(loinc_id, outcome)
                    loinc_scale = LoincScale.parse(loinc_scale)
                    outcome = Utility.parse_outcome(outcome)

                    annotation = LoincHpoAnnotation(loinc_id, loinc_scale, outcome, hpo_term, supplemental_term,
                                                    curators, comment)

                    if not ls:
                        if loinc_id in annotations:
                            if annotation in annotations[loinc_id]:
                                raise LoincHpoParsingError("Duplicate annotation exists for loinc code.")
                            else:
                                annotations[loinc_id].append(annotation)
                        else:
                            annotations[loinc_id] = [annotation]
                    else:
                        annotations.append(annotation)

                if not ls:
                    return AnnotationParser._build_query_map(annotations)
                else:
                    return annotations
        except (OSError, LoincHpoParsingError, LoincHpoValidationError) as e:
            raise e

    @staticmethod
    def parse_annotation(df):
        """ Parsing pandas dataframe to a traversable dictionary

            This method has a specific return of a dictionary of dictionaries to help us query or
            traverse annotations when resolving input queries from users.

        Args:
            df: A pandas dataframe with loinc queries. (loincId, loincMeasure, negated)

        Returns:
            A dictionary of dictionaries with a specific mapping to help us query for
            loinc code, measures and negations to get us an hpo code.

            {'29298-0': {'POS': {'N': 'HP:0009281'}}}

            In this example we see that a loinc_id has a measure of positive and it is not negated
            which results in an hpo code.

        Raises:
            OSError: When the file could not be opened or found.
            LoincHpoParsingError: When there were issues with file format
            LoincHpoValidationError: When there were issues with specific field expected values.
        """
        annotations = {}
        expected_fields = ("loincId", "loincScale", "outcome", "hpoTermId", "supplementalTermId", "curation", "comment")
        try:
            cols = df.columns
            if not all(col in expected_fields for col in cols):
                raise LoincHpoParsingError(
                    "Dataframe columns not correct, must be (loincId, loincScale,  outcome, " +
                    "hpoTermId,	supplmentalTermId, curator, comment)")
            for row in df.iterrows():
                loinc_id, loinc_scale, outcome, hpo_term, supplemental_term, curators, comment = row[1].tolist()

                loinc_scale = LoincScale.parse(loinc_scale)
                Utility.check_all(loinc_id, outcome)
                outcome = Utility.parse_outcome(outcome)

                annotation = LoincHpoAnnotation(loinc_id, loinc_scale, outcome, hpo_term, supplemental_term, curators,
                                                comment)
                if loinc_id in annotations:
                    if annotation in annotations[loinc_id]:
                        raise LoincHpoParsingError("Duplicate annotation exists for loinc code.")
                    else:
                        annotations[loinc_id].append(annotation)
                else:
                    annotations[loinc_id] = [annotation]

            return AnnotationParser._build_query_map(annotations)
        except (OSError, LoincHpoParsingError, LoincHpoValidationError) as e:
            raise e

    @staticmethod
    def _build_query_map(annotations):
        """
            remapping to a queryable dictionary given specific outcomes.
            loinc_id -> scale -> outcome -> negated -> term
        """
        query_map = {}
        for loinc_id in annotations:
            # get the list
            annotation_list = annotations[loinc_id]
            for annotation in annotation_list:
                if annotation.loinc_id not in query_map:
                    query_map[annotation.loinc_id] = {annotation.outcome: annotation.hpo_term}
                else:
                    if annotation.outcome not in query_map[annotation.loinc_id]:
                        query_map[annotation.loinc_id][annotation.outcome] = annotation.hpo_term
        return query_map



import pyspark.sql.dataframe

from loinchpo.model.ClinicalTableColumns import ClinicalTableColumns
from loinchpo import ClinicalTableName
from loinchpo.error.ClinicalParsingError import ClinicalParsingError


class ClinicalTableParser:

    def parse_table(self, path, table_name: ClinicalTableName, spark_session: pyspark.sql.SparkSession,
                    sep=',') -> pyspark.sql.DataFrame:
        """
            Parses a OMOP table to a spark dataframe

            Args:
                path: the path to the delimited omop table file
                table_name: a :class::`ClinicalTableName` of the csv being passed
                spark_session: a valid spark session
                sep: defaults to comma the seperator for the delimited file
            Returns:
                A spark dataframe representing the OMOP Clinical Table

            Raises:
                ClinicalParsingError: If the file is not formatted correctly or is missing required columns
        """
        if isinstance(path, str) and spark_session is not None:
            try:
                df = spark_session.read.csv(path, header=True, sep=sep)
                return self.verify_table(df, table_name)
            except Exception as e:
                if isinstance(e, ClinicalParsingError):
                    raise e
                else:
                    raise ClinicalParsingError(str(e))

    def verify_table(self, df: pyspark.sql.DataFrame, table_name: ClinicalTableName):
        """
            Verifies that a pyspark OMOP dataframe has the required columns to do transformation

            Args:
                df: A ::class::`pyspark.sql.Dataframe` of an OMOP table
                table_name: the name of the OMOP table for the dataframe
            Returns:
                The spark dataframe
            Raises:
                ClinicalParsingError: if the required columns are missing

        """
        if isinstance(df, pyspark.sql.dataframe.DataFrame):
            if self._required_columns_exist(df.columns, table_name):
                return df
            else:
                raise ClinicalParsingError(
                    "Some input columns %s are missing from required columns %s ".format(list(df.columns),
                                                                                         ClinicalTableColumns.get(
                                                                                             table_name))
                )

    def _required_columns_exist(self, input_columns, table_name):
        """
            Take input columns and table_name and returns whether we have the required columns to do our transform

            Args:
                input_columns: the columns of our input table
                table_name: A ClinicalTableName enum of the table

            Returns:
                Boolean of whether the required columns exist.
        """
        input_columns = [x.lower() for x in list(input_columns)]
        required_columns = ClinicalTableColumns.get(table_name)
        mutually_exclusive = ClinicalTableColumns.get_mutually_exclusive(table_name)
        if len(mutually_exclusive) > 0:
            return all(x in input_columns for x in required_columns) and any(x in input_columns for x in mutually_exclusive)
        return all(x in input_columns for x in required_columns)



from loinchpo.error.LoincHpoFileValidationError import LoincHpoValidationError


class QueryResolver:
    """ Resolve annotation queries to hpo codes

        Example:
            annotations = AnnotationParser.parse_annotation_file_dict(file_path)
            resolver = AnnotationResolver(annotations)
            query = Query(loinc_id, outcome)
            single_hpo_code = resolver.resolve(query)


        Attributes:
            annotations: A dictionary of dictionaries from AnnotationParser.

    """
    def __init__(self, annotations):
        self.annotations = annotations

    def resolve(self, query):
        """ Resolving a single query to a hpo code

        Args:
            query: An instance of Query, which contains the query with proper values.

        Returns:
            A single string hpo code or empty string if no annotation found.
        """

        try:
            return self.annotations[query.loinc_id][query.outcome]
        except (KeyError, LoincHpoValidationError):
            return ""



from enum import Enum, auto


class LoincScale(Enum):
    """ An enumeration representation for the possible loinc scale types.

    """
    QN = 1, "qn"
    ORD = 2, "ord"
    ORDQN = 3, "ordqn"
    NOM = 4, "nom"
    NAR = 5, "nar"
    MULTI = 6, "multi"
    DOC = 7, "doc"
    SET = 8, "set"
    UNKNOWN = auto()

    @staticmethod
    def parse(loinc_scale):
        """Mapping loinc scale to enumeration.

        Args:
            loinc_scale: A parsed string loinc scale from LoincHpoAnnotation file.

        Returns:
            An enumeration representation of loinc_scale or enumeration unknown for those that
            could not be mapped to an associated enumeration.
        """
        try:
            return LoincScale[loinc_scale.upper()]
        except (KeyError, AttributeError):
            return LoincScale.UNKNOWN

    @staticmethod
    def infer_long(long_name: str):
        """
            Helper function to infer long_name from a list of synonyms
        """
        if long_name is not None:
            if "quantitative" in long_name and "ordinal" in long_name:
                return str(LoincScale.ORDQN)
            elif "ordinal" in long_name:
                return str(LoincScale.ORD)
            elif "quantitative" in long_name:
                return str(LoincScale.QN)
            elif "nominal" in long_name or "qualitative" in long_name:
                return str(LoincScale.NOM)
            elif "narrative" in long_name:
                return str(LoincScale.NAR)
            elif "doc" in long_name:
                return str(LoincScale.DOC)
            elif "set" in long_name:
                return str(LoincScale.SET)
            elif "multi" in long_name:
                return str(LoincScale.MULTI)
            elif "panel" in long_name or "pnl" in long_name or "panl" in long_name:
                return str(LoincScale.SET)
        return str(LoincScale.UNKNOWN)

    def __str__(self):
        if self == LoincScale.QN:
            return "QN"
        elif self == LoincScale.ORD:
            return "ORD"
        elif self == LoincScale.ORDQN:
            return "ORDQN"
        elif self == LoincScale.NOM:
            return "NOM"
        elif self == LoincScale.NAR:
            return "NAR"
        elif self == LoincScale.MULTI:
            return "MULTI"
        elif self == LoincScale.DOC:
            return "DOC"
        elif self == LoincScale.SET:
            return "SET"
        else:
            return "UNKNOWN"



import pyspark.sql.functions as F
from pyspark.sql.functions import coalesce

from loinchpo import ClinicalTableName
from loinchpo import ClinicalTableParser
from loinchpo.core.Cleaner import Cleaner
from loinchpo.error.ClinicalParsingError import ClinicalParsingError


class MeasurementTransformer:

    def __init__(self):
        self.postive_category_indicators = ["positive", "detected", "abnormal", "yes"]
        self.negative_category_indicators = ["negative", "not detected", "normal", "acceptable", "healthy", "in range",
                                             "non-reactive", "nonreactive", "none detected", "none seen",
                                             "not detected/negative", "not present", "no"]

    def transform(self, measurement_table, concept_table):
        """
            We want to interpret measurement values to high low normal
            based on reference ranges and gather other values.

            Args:
                measurement_table: the measurement table dataframe
                concept_table: the concept table dataframe
                concept_synonym_table: the concept synonym dataframe
        """
        try:
            parser = ClinicalTableParser()
            parser.verify_table(measurement_table, ClinicalTableName.MEASUREMENT)
            parser.verify_table(concept_table, ClinicalTableName.CONCEPT)
        except ClinicalParsingError as e:
            print(str(e))
            exit(2)

        clean_df = self._process_measurement(measurement_table, concept_table)
        clean_df = self._obtain_loinc_results(clean_df)
        return clean_df

    def _process_measurement(self, measurement_table, concept_table):
        """Filters the measurement table to only include loinc measurements with a result"""

        loinc_concept_table = concept_table.filter((concept_table.domain_id == "Measurement") &
                                                   (concept_table.vocabulary_id == "LOINC") &
                                                   (concept_table.invalid_reason.isNull()))
        correct_measurement_val_col = "value_as_concept_name"
        correct_measurement_val_col = "value_as_concept_id" if "value_as_concept_name" not in measurement_table.columns else correct_measurement_val_col

        # Get measurement concept
        merged = measurement_table.withColumnRenamed("measurement_concept_id", "concept_id").filter(
            coalesce("value_as_number", correct_measurement_val_col).isNotNull()).join(loinc_concept_table,
                                                                                       ["concept_id"])

        # Figure out if we already have the column ( n3c auto joined these)
        if correct_measurement_val_col == "value_as_concept_id":
            # Get value as concept name
            value_concept = merged.select("value_as_concept_id").dropDuplicates()
            value_concept = value_concept.join(concept_table,
                                               value_concept.value_as_concept_id == concept_table.concept_id,
                                               "left").select("value_as_concept_id", "concept_name").withColumnRenamed(
                "concept_name", "value_as_concept_name")
            merged = merged.join(value_concept, "value_as_concept_id", "left")

        # remove invalid concepts that are not loinc measurements
        merged = merged.select("person_id", "visit_occurrence_id",
                               "measurement_id", "measurement_date", "concept_id",
                               "concept_code", "concept_name", "value_as_concept_name",
                               "value_as_number", "range_high", "range_low").dropDuplicates()

        # cast numeric columns to double
        merged = merged.withColumn("value_as_number", F.col("value_as_number").cast("double"))
        merged = merged.withColumn("range_low", F.col("range_low").cast("double"))
        merged = merged.withColumn("range_high", F.col("range_high").cast("double"))
        # verify proper handling of negative value_source_value
        return Cleaner.impute_measurement_values(merged)

    def _obtain_loinc_results(self, df):
        """
            Obtains the result of measurement and reference ranges

            Args:
                A cleaned measurement concept table of loinc values.
        """
        helper_function = F.udf(lambda x: self._infer_result(x[0], x[1], x[2], x[3]))
        input_columns = F.struct("value_as_concept_name", "value_as_number", "range_low", "range_high")
        return df.select("*", helper_function(input_columns).alias("loinc_result")).dropDuplicates()

    def _infer_result(self, value_as_concept_name, value_as_number, range_low, range_high):
        """Helper function that parses and returns the loinc result based on a reference range"""

        # Determine if a concept is numerical or categorical and check result.
        if value_as_concept_name is not None and value_as_number is None:
            if value_as_concept_name != "No matching concept":
                temp_result = self._checks_categorical_result_value(value_as_concept_name)
                if temp_result is not None:
                    result = temp_result
                else:
                    result = "Categorical result not a Current Mapped Type and numeric result value is Missing"
            else:
                result = "Categorical Result has value 'No matching concept' and numeric result value is Missing"
        elif value_as_concept_name is not None and value_as_number is not None:
            if value_as_concept_name != "No matching concept":
                temp_result = self._checks_categorical_result_value(value_as_concept_name)
                if temp_result is not None:
                    result = temp_result
                else:
                    result = self._checks_numeric_result_value(value_as_number, range_low, range_high)
            else:
                temp_result = self._checks_numeric_result_value(value_as_number, range_low, range_high)
                if temp_result == "missing both range_low and range_high values":
                    result = "Categorical Result has value 'No matching concept' and reference range values are missing"
                else:
                    result = temp_result
        elif value_as_concept_name is None and value_as_number is not None:
            result = self._checks_numeric_result_value(value_as_number, range_low, range_high)
        else:
            result = "Categorical Result is missing and numeric result is missing"

        return result

    def _checks_categorical_result_value(self, value_as_concept_name):
        """
            Verifies values for a categorical result from the value_as_concept_name column
        """

        if value_as_concept_name.lower() in self.postive_category_indicators + self.negative_category_indicators:
            if value_as_concept_name.lower() in self.postive_category_indicators:
                result = "positive"
            else:
                result = "negative"
        else:
            result = None
        return result

    def _checks_numeric_result_value(self, value_as_number, range_low, range_high):
        """Verifies values for a numeric result from the value_as_number column"""

        if range_low is not None and range_high is not None:
            if range_low != 0.0 and range_high != 0.0:
                if range_low < range_high:
                    if value_as_number < range_low:
                        result = "low"
                    elif value_as_number > range_high:
                        result = "high"
                    else:
                        result = "normal"
                elif range_low > range_high:
                    result = "range_low value > range_high value"
                else:
                    result = "range_low value == range_high value"
            elif range_low != 0.0 and range_high == 0.0:
                result = "range_high value == zero, but range_low value != 0"
            elif range_low == 0.0 and range_high != 0.0:
                result = "range_low value == zero, but range_high value != 0"
            else:
                result = "Both range_low and range_high values are zero"
        elif range_low is None and range_high is not None:
            result = "missing range_low value, but not missing range_high value"
        elif range_low is not None and range_high is None:
            result = "missing range_high value, but not missing range_low value"
        else:
            result = "missing both range_low and range_high values"
        return result



import csv
from loinchpo import Query
from loinchpo.error.LoincHpoFileValidationError import LoincHpoValidationError
from loinchpo.error.LoincHpoParsingError import LoincHpoParsingError


class QueryFileParser:
    """Provides methods for parsing query files

    """

    @staticmethod
    def parse(file_path):
        """ Parses query files to Query objects.

        Args:
            file_path: A path to a query file.

        Returns:
            A list of Query objects which have valid loinc_id and outcome values.
            [Query, Query]

        Raises:
            OSError: When the file could not be opened or found.
            LoincHpoParsingError: When there were issues with file format
            LoincValidationValidationError: When there were issues with specific field expected values.

        """
        queries = []
        try:
            with open(file_path) as f:
                reader = csv.reader(f, delimiter='\t')
                for line in reader:
                    loinc_id, outcome = line
                    queries.append(Query(loinc_id, outcome))
            return queries
        except (OSError, LoincHpoParsingError, LoincHpoValidationError) as e:
            raise e



from .io.Utility import Utility
from .model.Query import Query
from .model.LoincId import LoincId
from .model.LoincScale import LoincScale
from .model.ClinicalTableName import ClinicalTableName
from loinchpo.io.ClinicalTableParser import ClinicalTableParser
from .io.AnnotationParser import AnnotationParser
from .io.QueryFileParser import QueryFileParser
from .core.QueryResolver import QueryResolver
from .core.ConceptSynonymTransformer import ConceptSynonymTransformer
from .core.MeasurementTransformer import MeasurementTransformer
from .core.OMOPTransformer import OMOPTransformer




from loinchpo.error.LoincHpoFileValidationError import LoincHpoValidationError


class Utility:
    """A utility class for helping to parse LoincHpoAnnotation

    This class contains static methods to validate or map fields relevant to a LoincHpoAnnotation
    file.

    """

    @staticmethod
    def is_loinc_id(loinc_id):
        """Determines if the argument is a valid loinc id.

        Args:
            loinc_id: A string loinc id.

        Returns:
            Boolean if we are able to validate that string is similar to a loinc id
            (####-##)
        """
        try:
            loinc_pieces = loinc_id.split("-")
        except Exception:
            return False
        return loinc_pieces[0].isdigit() and loinc_pieces[1].isdigit()

    @staticmethod
    def is_outcome(outcome):
        """Determines if the argument is a valid measure value.

        Args:
            outcome: An observed outcome for a loinc test.

        Returns:
            Boolean value is in allowed values for provided outcome.
        """
        return outcome is not None and outcome.upper() in {"NEG", "POS", "H", "L", "N", "NEGATIVE", "POSITIVE",
                                                           "HIGH", "LOW", "NORMAL"}

    @staticmethod
    def parse_outcome(outcome):
        outcome = outcome.upper()
        if outcome == "NEG" or outcome == "NEGATIVE":
            return "NEG"
        elif outcome == "POS" or outcome == "POSITIVE":
            return "POS"
        elif outcome == "H" or outcome == "HIGH":
            return "H"
        elif outcome == "L" or outcome == "LOW":
            return "L"
        elif outcome == "N" or outcome == "NORMAL":
            return "N"

    @staticmethod
    def check_all(loinc_id, outcome):
        """Checks both loinc_id, loinc_scale, outcome for consistency.

        Based on observable negation values we return its boolean mapping.

        Args:
            loinc_id: A string loinc id
            loinc_scale: A LoincScale enumeration
            outcome: A string for the interpreted outcome for the loinc test

        Raises:
            LoincHpoValidationError: An error while validating or mapping the loinc id or observed
            measure
        """
        if not Utility.is_loinc_id(loinc_id):
            raise LoincHpoValidationError("Loinc Id {0} is not formatted properly `#-#`".
                                          format(loinc_id))
        elif not Utility.is_outcome(outcome):
            raise LoincHpoValidationError("Invalid outcome for Loinc Id {0} with value '{1}' must be "
                                          "one of [NEGATIVE, NEG], [POS, POSITIVE], [H, HIGH], [L,LOW], [N,NORMAL]."
                                          .format(loinc_id, outcome))



from setuptools import setup, find_packages

# read requirements/dependencies
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

# read description/README.md
with open("README.md", 'r') as fh:
    long_description = fh.read()


setup(name='loinchpo',
      version='1.1.2.dev1',
      packages=find_packages(exclude=["*tests*"]),
      install_requires=requirements,
      data_files=[('', ['requirements.txt', 'LICENSE.txt'])],
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Michael Gargano',
      author_email='michael.gargano@jax.com',
      url='https://github.com/TheJacksonLaboratory/loinchpo',
      description='Python version of LOINC2HPO',
      license='GPLv3',
      keywords='python, loinc, hpo, clinical, omop, human phenotype ontology',
      entry_points={'console_scripts': [
          'loinchpo = loinchpo.main:cli'
      ]})



from loinchpo.model.ClinicalTableName import ClinicalTableName


class ClinicalTableColumns:

    @staticmethod
    def get(table_name: ClinicalTableName):
        """
            Gets the required columns based on the ::class::`ClinicalTableName`.
        """
        if table_name == ClinicalTableName.MEASUREMENT:
            return ['measurement_id', 'measurement_concept_id', 'person_id', 'visit_occurrence_id',
                    'measurement_date', 'value_as_number', 'range_low', 'range_high']
        elif table_name == ClinicalTableName.CONCEPT:
            return ['concept_id', 'concept_code', 'concept_name', 'vocabulary_id', 'concept_class_id',
                    'domain_id', 'invalid_reason']
        elif table_name == ClinicalTableName.CONCEPT_SYNONYM:
            return ['concept_id', 'concept_synonym_name']
        elif table_name == ClinicalTableName.VOCABULARY:
            return ['vocabulary_id', 'vocabulary_version']

    @staticmethod
    def get_mutually_exclusive(table_name: ClinicalTableName):
        if table_name == ClinicalTableName.MEASUREMENT:
            return ['value_as_concept_id', 'value_as_concept_name']
        return []



from loinchpo.error.LoincHpoParsingError import LoincHpoParsingError


class LoincId:
    """
        A class to represent a loinc id

    """
    def __init__(self, loinc_id):
        self.num, self.suffix, self.loinc_id = self._parse_loinc_id(loinc_id)

    @staticmethod
    def _parse_loinc_id(loinc_id):
        try:
            num, suffix = loinc_id.split("-")
            return num, suffix, loinc_id
        except (ValueError, AttributeError):
            raise LoincHpoParsingError("Malformed Loinc code {0}".format(loinc_id))
